/// <reference types="node" />
import { RpsContext } from 'rpscript-interface';
import { EventEmitter } from 'events';
/** Basic utility for rpscript.
 * RPScript make heavy use of ramda for utility and data manipulation. A lot of the keywords here are a direct port from ramda.
 * Besides ramda's methods, there are also utility for event listening, variable assignment, terminal print etc.
 * @namespace Basic
 *
 * @example
 * rps install basic
*/
export default class RPSBasic {
    /**
     * @function log
     * @memberof Basic
     * @example
     * ;print 'Hello'
     * log 'Hello' 'World'
     * ;print 'Hello' again
     * log $RESULT
     *
     * @param {List} text information to be printed out on the terminal.
     * @returns {*}  Similar to text input.
     * @summary log :: ...a → a
     * @description
     * This is a wrapper for javascript console.log function.
     *
     * @see {@link https://www.w3schools.com/jsref/met_console_log.asp}
     *
    */
    print(ctx: RpsContext, opts: {}, ...text: any[]): Promise<any>;
    /**
   * @function stdout
   * @memberof Basic
   * @example
   * ;print 'Hello'
   * stdout 'Hello'
   *
   * @param {string} text information to be printed out on the terminal.
   * @returns {*}  Similar to text input.
   * @summary stdout :: a → a
   *
   * @see {@link https://nodejs.org/api/process.html#process_process_stdout}
   *
  */
    stdout(ctx: RpsContext, opts: {}, text?: any): Promise<any>;
    /**
   * @function as
   * @memberof Basic
   * @example
   * as 'varName' 1
   * ;print out the value 1
   * console-log $varName
   *
   * read 'filename.txt' | as 'varName'
   * ;this will print out the content of 'filename.txt'
   * console-log $varName
   *
   * @param {string} variable Variable name.
   * @param {*} value  Value to be assigned to the variable.
   * @returns {*}  Value of the variable.
   * @summary as :: String → a → a
   * @description
   * This is equivalent to variable assignment in programming language.
   * The assigned variable can be access by prefixing $ on the variable name.
   *
   *
  */
    as(ctx: RpsContext, opts: {}, variable: string, value: any): Promise<any>;
    /**
     * @function assign
     * @memberof Basic
     *
     * @param {string} variable Variable name.
     * @param {*} value  Value to be assigned to the variable.
     * @returns {*}  Value of the variable.
     * @summary assign :: String → a → a
     *
     *
    */
    assign(ctx: RpsContext, opts: {}, variable: string, value: any): Promise<any>;
    /**
   * @function listen-once
   * @memberof Basic
   * @example
   * listen-once 'connected' $emitter
   *
   * @param {EventEmitter} event The object to listen to.
   * @param {string} eventName Name to listen for event.
   * @returns {*}  If condition is met, result of exec. else null.
   * @summary listen-once :: EventEmitter → String → a
   *
   * @see {@link https://nodejs.org/api/events.html#events_emitter_once_eventname_listener}
   *
  */
    listenOnce(ctx: RpsContext, opts: {}, event: EventEmitter, evtName: string): Promise<any>;
    /**
     * @function listen-on
     * @memberof Basic
     * @example
     * listen-on 'start' (($output)=>log $output) $emitter
     *
     * @param {EventEmitter} event The object to listen to.
     * @param {string} eventName Name to listen for event.
     * @returns {*}  If condition is met, result of exec. else null.
     * @summary listen-on :: String → (...* → *) → EventEmitter → EventEmitter
     *
     * @see {@link https://nodejs.org/api/events.html#events_emitter_once_eventname_listener}
     *
    */
    on(ctx: RpsContext, opts: {}, evtName: string, cb: (any) => void, event: EventEmitter): Promise<EventEmitter>;
    /***
     * @function exit
     * @memberof Basic
     * @example
     * exit
     *
     * @param {number} code Optional exit code
     *
    */
    exit(ctx: RpsContext, opts: {}, code: number): Promise<void>;
    /***
   * @function wait
   * @memberof Basic
   * @example
   * ;wait for 5 second
   * wait 5
   *
   * @param {number} period Period to wait for in second.
   * @param {*} output returned output after period second.
   * @returns {*} Previous result.
   * @summary wait :: Number → a → a
   *
  */
    wait(ctx: RpsContext, opts: {}, ...params: any[]): Promise<any>;
    /**
     * @function stringify
     * @memberof Basic
     * @example
     * stringify {'a':1,'b':2}
     *
     * @param {Object} object
     * @returns {string} String result
     * @summary stringify :: a → String
     *
    */
    stringify(ctx: RpsContext, opts: {}, obj: any): Promise<string>;
    /**
     * @function abs
     * @memberof Basic
     * @example
     * log abs -5.1
     *
     * @summary abs :: Number → Number
     *
    */
    abs(ctx: RpsContext, opts: {}, ...args: number[]): Promise<number | Function>;
    /**
     * @function ceil
     * @memberof Basic
     * @example
     * ;ceil value
     * log ceil 5.1
     *
     * @summary ceil :: Number → Number
     *
    */
    ceil(ctx: RpsContext, opts: {}, ...args: number[]): Promise<number | Function>;
    /**
     * @function math-max
     * @memberof Basic
     * @example
     * max 5.1 1.2 3.3
     *
     *
     * @summary math-max :: Number → ...Number → Number
     *
    */
    mathMax(ctx: RpsContext, opts: {}, ...num: number[]): Promise<number>;
    /**
     * @function math-min
     * @memberof Basic
     * @example
     * ;min value
     * min 5.1 1.2 3.3
     *
     *
     * @summary math-min :: Number → ...Number → Number
     *
    */
    mathMin(ctx: RpsContext, opts: {}, ...num: number[]): Promise<number>;
    /**
     * @function floor
     * @memberof Basic
     * @example
     * ;floor value
     * floor 5.1
     *
     * @param {number} number
     * @returns {number} number.
     *
     * @summary floor :: Number → Number
     *
    */
    floor(ctx: RpsContext, opts: {}, ...num: number[]): Promise<number>;
    /**
     * @function power
     * @memberof Basic
     * @example
     * ;power value
     * power 5 3
     *
     * @param {number} x
     * @param {number} y
     * @returns {number} number.
     *
     * @summary pow :: Number → Number → Number
     *
    */
    power(ctx: RpsContext, opts: {}, ...args: number[]): Promise<number | Function>;
    /**
     * @function random
     * @memberof Basic
     * @example
     * ;random
     * random
     *
     * @returns {number} number.
     *
     * @summary random :: Number
     *
    */
    random(ctx: RpsContext, opts: {}): Promise<number>;
    /**
     * @function round
     * @memberof Basic
     * @example
     * log round 1.3
     *
     * @summary round :: Number → Number
     *
    */
    round(ctx: RpsContext, opts: {}, num: number): Promise<number>;
    /**
     * @function truncate
     * @memberof Basic
     * @example
     * truncate 1.3
     *
     * @summary truncate :: Number → Number
     *
    */
    trunc(ctx: RpsContext, opts: {}, num: number): Promise<number>;
    /**************************** Ramda (List) ***********************************/
    /**
     * @function adjust
     * @memberof Basic
     * @example
     * adjust (add 10) 1 [1,2,3]
     *
     * @summary adjust :: (a → a) → Number → [a] → [a]
     *
     * @see {@link https://ramdajs.com/docs#adjust}
    */
    adjust(ctx: RpsContext, opts: {}, ...params: any[]): Promise<any>;
    /**
   * @function all
   * @memberof Basic
   * @example
   * log all (equals 3) [3, 3, 3, 3]
   *
   * @summary all :: (a → Boolean) → [a] → Boolean
   *
   * @see {@link https://ramdajs.com/docs#all}
  */
    all(ctx: RpsContext, opts: {}, ...params: any[]): Promise<any>;
    /**
     * @function any
     * @memberof Basic
     * @example
     * log any (equals 1) [1,2]
     *
     * @summary any :: (a → Boolean) → [a] → Boolean
     *
     * @see {@link https://ramdajs.com/docs#any}
    */
    any(ctx: RpsContext, opts: {}, ...params: any[]): Promise<any>;
    /**
     * @function aperture
     * @memberof Basic
     * @example
     * log aperture 2 [1,2,3,4,5]
     *
     * @summary aperture :: Number → [a] → [[a]]
     *
     * @see {@link https://ramdajs.com/docs#aperture}
    */
    aperture(ctx: RpsContext, opts: {}, ...params: any[]): Promise<any>;
    /**
     * @function append
     * @memberof Basic
     * @example
     * log append "tests" ["hello"]
     *
     * @summary append :: a → [a] → [a]
     *
     * @see {@link https://ramdajs.com/docs#append}
    */
    append(ctx: RpsContext, opts: {}, ...params: any[]): Promise<any>;
    /**
     * @function chain
     * @memberof Basic
     *
     * @summary chain :: Chain m => (a → m b) → m a → m b
     *
     * @see {@link https://ramdajs.com/docs#chain}
    */
    chain(ctx: RpsContext, opts: {}, ...params: any[]): Promise<any>;
    /**
     * @function concat
     * @memberof Basic
     * @example
     * log concat "hello" "world"
     *
     * @summary concat :: [a] → [a] → [a]
     *
     * @see {@link https://ramdajs.com/docs#concat}
    */
    concat(ctx: RpsContext, opts: {}, ...params: any[]): Promise<any>;
    /**
     * @function contains
     * @memberof Basic
     * @example
     * log contains 3 [1,2,3]
     *
     * @summary contains :: a → [a] → Boolean
     *
     * @see {@link https://ramdajs.com/docs#contains}
    */
    contains(ctx: RpsContext, opts: {}, ...params: any[]): Promise<any>;
    /**
     * @function drop
     * @memberof Basic
     * @example
     * log drop 1 [1,2,3]
     *
     * @summary drop :: Number → [a] → [a]
     *
     * @see {@link https://ramdajs.com/docs#drop}
    */
    drop(ctx: RpsContext, opts: {}, ...params: any[]): Promise<any>;
    /**
     * @function drop-last
     * @memberof Basic
     * @example
     * log drop-last 1 [1,2,3]
     *
     * @summary drop-last :: Number → [a] → [a]
     *
     * @see {@link https://ramdajs.com/docs#dropLast}
    */
    dropLast(ctx: RpsContext, opts: {}, ...params: any[]): Promise<any>;
    /**
     * @function drop-last-while
     * @memberof Basic
     *
     * @summary drop-last-while :: (a → Boolean) → [a] → [a]
     *
     * @see {@link https://ramdajs.com/docs#dropLastWhile}
    */
    dropLastWhile(ctx: RpsContext, opts: {}, ...params: any[]): Promise<any>;
    /**
     * @function drop-repeats
     * @memberof Basic
     * @example
     * log drop-repeats [1,1,1,2,3,4,4,4,2,2]
     *
     * @summary drop-repeats :: [a] → [a]
     *
     * @see {@link https://ramdajs.com/docs#dropRepeats}
    */
    dropRepeats(ctx: RpsContext, opts: {}, ...params: any[]): Promise<any>;
    /**
     * @function drop-repeats-with
     * @memberof Basic
     * @example
     * log drop-repeats-with (eq-by abs) [1,-1,3,-4,-4]
     *
     * @summary drop-repeats-with :: ((a,a) → Boolean) → [a] → [a]
     *
     * @see {@link https://ramdajs.com/docs#dropRepeatsWith}
    */
    dropRepeatsWith(ctx: RpsContext, opts: {}, ...params: any[]): Promise<any>;
    /**
     * @function drop-while
     * @memberof Basic
     * @example
     * log drop-while (eq-by abs) [1,-1,3,-4,-4]
     *
     * @summary drop-while :: (a → Boolean) → [a] → [a]
     *
     * @see {@link https://ramdajs.com/docs#dropWhile}
    */
    dropWhile(ctx: RpsContext, opts: {}, ...params: any[]): Promise<any>;
    /**
     * @function ends-with
     * @memberof Basic
     * @example
     * log ends-with 'c' 'abc'
     *
     * @summary ends-with :: [a] → Boolean
     *
     * @see {@link https://ramdajs.com/docs#endsWith}
    */
    endsWith(ctx: RpsContext, opts: {}, ...params: any[]): Promise<any>;
    /**
     * @function filter
     * @memberof Basic
     * @example
     * log filter (lt 2) [1,2,3]
     *
     * @summary filter :: Filterable f => (a → Boolean) → f a → f a
     *
     * @see {@link https://ramdajs.com/docs#filter}
    */
    filter(ctx: RpsContext, opts: {}, ...params: any[]): Promise<any[]>;
    /**
     * @function find
     * @memberof Basic
     * @example
     * find (prop-eq 'a' 2) [{'a':1},{'a':2},{'a':3}]
     *
     * @summary find :: (a → Boolean) → [a] → a | undefined
     *
     * @see {@link https://ramdajs.com/docs#find}
    */
    find(ctx: RpsContext, opts: {}, ...params: any[]): Promise<any>;
    /**
     * @function find-index
     * @memberof Basic
     * @example
     * find-index (prop-eq 'a' 2) [{'a':1},{'a':2},{'a':3}]
     *
     * @summary find-index :: (a → Boolean) → [a] → Number
     *
     * @see {@link https://ramdajs.com/docs#findIndex}
    */
    findIndex(ctx: RpsContext, opts: {}, ...params: any[]): Promise<any>;
    /**
     * @function find-last
     * @memberof Basic
     * @example
     * find-last (prop-eq 'a' 2) [{'a':1},{'a':2},{'a':3}]
     *
     * @summary find-last :: (a → Boolean) → [a] → a | undefined
     *
     * @see {@link https://ramdajs.com/docs#findLast}
    */
    findLast(ctx: RpsContext, opts: {}, ...params: any[]): Promise<any>;
    /**
     * @function find-last-index
     * @memberof Basic
     * @example
     * find-last-index (prop-eq 'a' 2) [{'a':1},{'a':2},{'a':3}]
     *
     * @summary find-last-index :: (a → Boolean) → [a] → Number
     *
     * @see {@link https://ramdajs.com/docs#findLastIndex}
    */
    findLastIndex(ctx: RpsContext, opts: {}, ...params: any[]): Promise<any>;
    /**
     * @function flatten
     * @memberof Basic
     * @example
     * log flatten [1,2,[3,4]]
     *
     * @summary flatten :: [a] → [b]
     *
     * @see {@link https://ramdajs.com/docs#flatten}
    */
    flatten(ctx: RpsContext, opts: {}, ...params: any[]): Promise<any>;
    /**
     * @function for-each
     * @memberof Basic
     * @example
     * for-each (log) [1,2,3,4]
     *
     * @summary for-each :: (a → *) → [a] → [a]
     *
     * @see {@link https://ramdajs.com/docs#forEach}
    */
    forEach(ctx: RpsContext, opts: {}, ...params: any[]): Promise<any>;
    /**
     * @function from-pairs
     * @memberof Basic
     * @example
     * log from-pairs [['a',1],['b',2],['c',3]]
     *
     * @summary from-pairs :: [[k,v]] → {k:v}
     *
     * @see {@link https://ramdajs.com/docs#fromPairs}
    */
    fromPairs(ctx: RpsContext, opts: {}, ...params: any[]): Promise<any>;
    /**
     * @function group-by
     * @memberof Basic
     *
     * @summary group-by :: ((a → String) → [a] → {String:[a]}
     *
     * @see {@link https://ramdajs.com/docs#groupBy}
    */
    groupBy(ctx: RpsContext, opts: {}, ...params: any[]): Promise<any>;
    /**
     * @function group-with
     * @memberof Basic
     * @example
     * log group-with (equals) [0,1,1,2,3,4,6,4]
     *
     * @summary group-with :: ((a,a) → Boolean) → [a] → [[a]]
     *
     * @see {@link https://ramdajs.com/docs#groupWith}
    */
    groupWith(ctx: RpsContext, opts: {}, ...params: any[]): Promise<any>;
    /**
     * @function head
     * @memberof Basic
     * @example
     * log head ['ff',''aa','lol']
     *
     * @summary head :: [a] → a | undefined
     *
     * @see {@link https://ramdajs.com/docs#head}
    */
    head(ctx: RpsContext, opts: {}, ...params: any[]): Promise<any>;
    /**
     * @function index-of
     * @memberof Basic
     * @example
     * index-of 3 [1,2,3,4]
     *
     * @summary index-of :: a → [a] → Number
     *
     * @see {@link https://ramdajs.com/docs#indexOf}
    */
    indexOf(ctx: RpsContext, opts: {}, ...params: any[]): Promise<any>;
    /**
     * @function index-by
     * @memberof Basic
     * @example
     * index-by (prop 'id') [{'id':'abc','title':'A'},{'id':'xyz','title':'B'}]
     *
     * @summary index-by :: (a → String) → [{k:v}] → {:{k:v}}
     *
     * @see {@link https://ramdajs.com/docs#indexBy}
    */
    indexBy(ctx: RpsContext, opts: {}, ...params: any[]): Promise<any>;
    /**
     * @function init
     * @memberof Basic
     * @example
     * log init [1, 2, 3]
     *
     * @param {Array} array
     * @summary init :: [a] → [a]
     *
     * @see {@link https://ramdajs.com/docs#init}
    */
    init(ctx: RpsContext, opts: {}, ...params: any[]): Promise<any>;
    /**
     * @function insert
     * @memberof Basic
     * @example
     * insert 2 'x' [1,2,3,4]
     *
     * @summary insert :: Number → a → [a] → [a]
     *
     * @see {@link https://ramdajs.com/docs#insert}
    */
    insert(ctx: RpsContext, opts: {}, ...params: any[]): Promise<any>;
    /**
     * @function insert-all
     * @memberof Basic
     * @example
     * insert-all 2 ['x','y','z'] [1,2,3,4]
     *
     * @summary insert-all :: Number → [a] → [a] → [a]
     *
     * @see {@link https://ramdajs.com/docs#insertAll}
    */
    insertAll(ctx: RpsContext, opts: {}, ...params: any[]): Promise<any>;
    /**
     * @function intersperse
     * @memberof Basic
     * @example
     * intersperse 'n' ['x','y','z']
     *
     * @summary intersperse :: a → [a] → [a]
     *
     * @see {@link https://ramdajs.com/docs#intersperse}
    */
    intersperse(ctx: RpsContext, opts: {}, ...params: any[]): Promise<any>;
    /**
     * @function join
     * @memberof Basic
     * @example
     * join ' ' ['x','y','z']
     *
     * @summary join :: String → [a] → String
     *
     * @see {@link https://ramdajs.com/docs#join}
    */
    join(ctx: RpsContext, opts: {}, ...params: any[]): Promise<any>;
    /**
     * @function last
     * @memberof Basic
     * @example
     * log last ['x','y','z']
     *
     * @summary last :: [a] → a | undefined
     *
     * @see {@link https://ramdajs.com/docs#last}
    */
    last(ctx: RpsContext, opts: {}, ...params: any[]): Promise<any>;
    /**
     * @function last-index-of
     * @memberof Basic
     * @example
     * log last-index-of 'y' ['x','y','y','z']
     *
     * @summary last-index-of :: a → [a] → Number
     *
     * @see {@link https://ramdajs.com/docs#lastIndexOf}
    */
    lastIndexOf(ctx: RpsContext, opts: {}, ...params: any[]): Promise<any>;
    /**
     * @function length
     * @memberof Basic
     * @example
     * length [1,2,3]
     *
     * @summary length :: [a] → Number
     *
     * @see {@link https://ramdajs.com/docs/#length}
    */
    length(ctx: RpsContext, opts: {}, ...params: any[]): Promise<any>;
    /**
     * @function map
     * @memberof Basic
     * @example
     * log map (multiple 2) [1,2,3]
     *
     * @summary map :: Functor f => (a → b) → f a → f b
     *
     * @see {@link https://ramdajs.com/docs#map}
    */
    map(ctx: RpsContext, opts: {}, ...params: any[]): Promise<any[]>;
    /**
     * @function map-accum
     * @memberof Basic
     * @example
     * map-accum (multiple 2) [1,2,3]
     *
     * @summary map-accum :: ((acc,x) → (acc,y)) → acc → [x] → (acc, [y])
     *
     * @see {@link https://ramdajs.com/docs#mapAccum}
    */
    mapAccum(ctx: RpsContext, opts: {}, ...params: any[]): Promise<any>;
    /**
     * @function map-accum-right
     * @memberof Basic
     *
     * @summary map-accum-right :: ((acc,x) → (acc,y)) → acc → [x] → ([y],acc)
     *
     * @see {@link https://ramdajs.com/docs#mapAccumRight}
    */
    mapAccumRight(ctx: RpsContext, opts: {}, ...params: any[]): Promise<any>;
    /**
     * @function merge-all
     * @memberof Basic
     * @example
     * merge-all [{'foo':1},{'bar':2},{'baz':3}]
     *
     * @param {Array} list
     * @summary merge-all :: [{k:v}] → {k:v}
     *
     * @see {@link https://ramdajs.com/docs#mergeAll}
    */
    mergeAll(ctx: RpsContext, opts: {}, ...params: any[]): Promise<any>;
    /**
     * @function none
     * @memberof Basic
     *
     * @summary none :: (a → Boolean) → [a] → Boolean
     *
     * @see {@link https://ramdajs.com/docs#none}
    */
    none(ctx: RpsContext, opts: {}, ...params: any[]): Promise<any>;
    /**
     * @function nth
     * @memberof Basic
     * @example
     * nth 1 ['a','b','c']
     *
     * @summary nth :: Number → [a] → a | undefined
     *
     * @see {@link https://ramdajs.com/docs#nth}
    */
    nth(ctx: RpsContext, opts: {}, ...params: any[]): Promise<any>;
    /**
     * @function pair
     * @memberof Basic
     * @example
     * log pair 'foo' 'bar'
     *
     * @summary pair :: a → b → (a,b)
     *
     * @see {@link https://ramdajs.com/docs#pair}
    */
    pair(ctx: RpsContext, opts: {}, ...params: any[]): Promise<any>;
    /**
     * @function partition
     * @memberof Basic
     * @example
     * log partition (contains 's') ['sss','ttt','sas','abc']
     *
     * @summary partition :: Filterable f => (a → Boolean) → f a → [f a, f a]
     *
     * @see {@link https://ramdajs.com/docs#partition}
    */
    partition(ctx: RpsContext, opts: {}, ...params: any[]): Promise<any>;
    /**
     * @function pluck
     * @memberof Basic
     * @example
     * pluck (prop 'a') [{a:1},{a:2}]
     *
     * @summary pluck :: Functor f => k → f {k:v} → f v
     *
     * @see {@link https://ramdajs.com/docs#pluck}
    */
    pluck(ctx: RpsContext, opts: {}, ...params: any[]): Promise<any>;
    /**
     * @function prepend
     * @memberof Basic
     * @example
     * log prepend 'fee' ['fi','fo','fum']
     *
     * @summary prepend :: a → [a] → [a]
     *
     * @see {@link https://ramdajs.com/docs#prepend}
    */
    prepend(ctx: RpsContext, opts: {}, ...params: any[]): Promise<any>;
    /**
     * @function range
     * @memberof Basic
     * @example
     * log range 1 5
     *
     * @summary range :: Number → Number → [Number]
     *
     * @see {@link https://ramdajs.com/docs#range}
    */
    range(ctx: RpsContext, opts: {}, ...params: any[]): Promise<any>;
    /**
     * @function reduce
     * @memberof Basic
     * @example
     * log reduce (subtract) 0 [1,2,3,4]
     *
     * @summary reduce :: ((a,b) → a) → a → [b] → a
     *
     * @see {@link https://ramdajs.com/docs#reduce}
    */
    reduce(ctx: RpsContext, opts: {}, ...params: any[]): Promise<any>;
    /**
     * @function reduce-by
     * @memberof Basic
     *
     * @summary reduce-by :: ((a,b) → a) → a → (b → String) → [b] → {String:a}
     *
     * @see {@link https://ramdajs.com/docs#reduceBy}
    */
    reduceBy(ctx: RpsContext, opts: {}, ...params: any[]): Promise<any>;
    /**
     * @function reduce-right
     * @memberof Basic
     * @example
     * reduce-right (subtract) 0 [1,2,3,4]
     *
     * @summary reduce-right :: ((a,b) → b) → b → [a] → b
     *
     * @see {@link https://ramdajs.com/docs#reduceRight}
    */
    reduceRight(ctx: RpsContext, opts: {}, ...params: any[]): Promise<any>;
    /**
     * @function reduce-while
     * @memberof Basic
     *
     * @summary reduce-while :: ((a,b) → Boolean) → ((a,b) → a) → a → [b] → a
     *
     * @see {@link https://ramdajs.com/docs#reduceWhile}
    */
    reduceWhile(ctx: RpsContext, opts: {}, ...params: any[]): Promise<any>;
    /**
     * @function reject
     * @memberof Basic
     *
     * @summary reject :: Filterable f => (a → Boolean) → f a → f a
     *
     * @see {@link https://ramdajs.com/docs#reject}
    */
    reject(ctx: RpsContext, opts: {}, ...params: any[]): Promise<any>;
    /**
     * @function remove
     * @memberof Basic
     * @example
     * log remove 2 3 [1,2,3,4,5,6,7,8]
     *
     * @summary remove :: Number → Number → [a] → [a]
     *
     * @see {@link https://ramdajs.com/docs#remove}
    */
    remove(ctx: RpsContext, opts: {}, ...params: any[]): Promise<any>;
    /**
     * @function repeat
     * @memberof Basic
     * @example
     * log repeat "hi" 5
     *
     * @summary repeat :: a → n → [a]
     *
     * @see {@link https://ramdajs.com/docs#repeat}
    */
    repeat(ctx: RpsContext, opts: {}, ...params: any[]): Promise<any>;
    /**
     * @function reverse
     * @memberof Basic
     * @example
     * log reverse [1,2,3]
     *
     * @summary reverse :: [a] → [a]
     *
     * @see {@link https://ramdajs.com/docs#reverse}
    */
    reverse(ctx: RpsContext, opts: {}, ...params: any[]): Promise<any>;
    /**
     * @function scan
     * @memberof Basic
     * @example
     * log scan (multiply) 1 [1, 2, 3, 4]
     *
     * @summary scan :: ((a,b) → a) → a → [b] → [a]
     *
     * @see {@link https://ramdajs.com/docs#scan}
    */
    scan(ctx: RpsContext, opts: {}, ...params: any[]): Promise<any>;
    /**
     * @function slice
     * @memberof Basic
     * @example
     * log slice 1 3 ['a','b','c','d']
     *
     * @summary slice :: Number → Number → [a] → [a]
     *
     * @see {@link https://ramdajs.com/docs#slice}
    */
    slice(ctx: RpsContext, opts: {}, ...params: any[]): Promise<any>;
    /**
     * @function sort
     * @memberof Basic
     * @example
     * log sort (ascend prop "name") [{'name':'z'},{'name':'a'}]
     *
     * @summary sort :: ((a,a) → Number) → [a] → [a]
     *
     * @see {@link https://ramdajs.com/docs#sort}
    */
    sort(ctx: RpsContext, opts: {}, ...params: any[]): Promise<any>;
    /**
     * @function split-at
     * @memberof Basic
     * @example
     * split-at 1 [1,2,3]
     *
     * @summary split-at :: Number → [a] → [[a],[a]]
     *
     * @see {@link https://ramdajs.com/docs#splitAt}
    */
    splitAt(ctx: RpsContext, opts: {}, ...params: any[]): Promise<any>;
    /**
     * @function split-every
     * @memberof Basic
     * @example
     * split-every 3 [1,2,3,4,5,6,7]
     *
     * @summary split-every :: Number → [a] → [[a]]
     *
     * @see {@link https://ramdajs.com/docs#splitEvery}
    */
    splitEvery(ctx: RpsContext, opts: {}, ...params: any[]): Promise<any>;
    /**
     * @function split-when
     * @memberof Basic
     * @example
     * split-when (equals 3) [1,2,3,4,5,6,7]
     *
     * @summary split-when :: (a → Boolean) → [a] → [[a],[a]]
     *
     * @see {@link https://ramdajs.com/docs#splitWhen}
    */
    splitWhen(ctx: RpsContext, opts: {}, ...params: any[]): Promise<any>;
    /**
     * @function starts-with
     * @memberof Basic
     * @example
     * starts-with 'a' 'abc'
     *
     * @summary starts-with :: [a] → Boolean
     *
     * @see {@link https://ramdajs.com/docs#startsWith}
    */
    startsWith(ctx: RpsContext, opts: {}, ...params: any[]): Promise<any>;
    /**
     * @function tail
     * @memberof Basic
     * @example
     * log tail [1,2,3]
     *
     * @summary tail :: [a] → [a]
     *
     * @see {@link https://ramdajs.com/docs#tail}
    */
    tail(ctx: RpsContext, opts: {}, ...params: any[]): Promise<any>;
    /**
     * @function take
     * @memberof Basic
     * @example
     * log take 2 [1,2,3]
     *
     * @summary take :: [a] → [a]
     *
     * @see {@link https://ramdajs.com/docs#take}
    */
    take(ctx: RpsContext, opts: {}, ...params: any[]): Promise<any>;
    /**
     * @function take-last
     * @memberof Basic
     * @example
     * log take-last 2 [1,2,3]
     *
     * @summary take-last :: [a] → [a]
     *
     * @see {@link https://ramdajs.com/docs#takeLast}
    */
    takeLast(ctx: RpsContext, opts: {}, ...params: any[]): Promise<any>;
    /**
     * @function take-last-while
     * @memberof Basic
     *
     * @summary take-last-while :: (a → Boolean) → [a] → [a]
     *
     * @see {@link https://ramdajs.com/docs#takeLastWhile}
    */
    takeLastWhile(ctx: RpsContext, opts: {}, ...params: any[]): Promise<any>;
    /**
     * @function take-while
     * @memberof Basic
     *
     * @summary take-while :: (a → Boolean) → [a] → [a]
     *
     * @see {@link https://ramdajs.com/docs#takeWhile}
    */
    takeWhile(ctx: RpsContext, opts: {}, ...params: any[]): Promise<any>;
    /**
     * @function times
     * @memberof Basic
     * @example
     * times (identity) 5
     *
     * @summary times :: (Number → a) → Number → [a]
     *
     * @see {@link https://ramdajs.com/docs#times}
    */
    times(ctx: RpsContext, opts: {}, ...params: any[]): Promise<any>;
    /**
     * @function transpose
     * @memberof Basic
     * @example
     * log transpose [[1,'a'],[2,'b'],[3,'c']]
     *
     * @summary transpose :: [[a]] → [[a]]
     *
     * @see {@link https://ramdajs.com/docs#transpose}
    */
    transpose(ctx: RpsContext, opts: {}, ...params: any[]): Promise<any>;
    /**
     * @function unfold
     * @memberof Basic
     *
     * @summary unfold :: (a → [b]) → * → [b]
     *
     * @see {@link https://ramdajs.com/docs#unfold}
    */
    unfold(ctx: RpsContext, opts: {}, ...params: any[]): Promise<any>;
    /**
     * @function uniq
     * @memberof Basic
     * @example
     * uniq [1,2,3,3,4,5,4]
     *
     * @summary uniq :: [a] → [a]
     *
     * @see {@link https://ramdajs.com/docs#uniq}
    */
    uniq(ctx: RpsContext, opts: {}, ...params: any[]): Promise<any>;
    /**
     * @function uniq-by
     * @memberof Basic
     * @example
     * uniq-by (abs) [-1, -5, 2, 10, 1, 2]
     *
     * @summary uniq-by :: (a → b) → [a] → [a]
     *
     * @see {@link https://ramdajs.com/docs#uniqBy}
    */
    uniqBy(ctx: RpsContext, opts: {}, ...params: any[]): Promise<any>;
    /**
     * @function uniq-with
     * @memberof Basic
     *
     * @summary uniq-with :: ((a,a) → Boolean) → [a] → [a]
     *
     * @see {@link https://ramdajs.com/docs#uniqWith}
    */
    uniqWith(ctx: RpsContext, opts: {}, ...params: any[]): Promise<any>;
    /**
     * @function unnest
     * @memberof Basic
     * @example
     * unnest  [1 , [2], [[3]]]
     *
     * @summary unnest :: Chain c => c (c a) → c a
     *
     * @see {@link https://ramdajs.com/docs#unnest}
    */
    unnest(ctx: RpsContext, opts: {}, ...params: any[]): Promise<any>;
    /**
     * @function update
     * @memberof Basic
     * @example
     * update 1 11 [0,1,2]
     *
     * @summary update :: Number → a → [a] → [a]
     *
     * @see {@link https://ramdajs.com/docs#update}
    */
    update(ctx: RpsContext, opts: {}, ...params: any[]): Promise<any>;
    /**
     * @function without
     * @memberof Basic
     * @example
     * log without [1,2] [1,2,1,3,4]
     *
     * @summary without :: [a] → [a] → [a]
     *
     * @see {@link https://ramdajs.com/docs#without}
    */
    without(ctx: RpsContext, opts: {}, ...params: any[]): Promise<any>;
    /**
     * @function xprod
     * @memberof Basic
     * @example
     * log xprod [1,2] ['a','b']
     *
     * @summary xprod :: [a] → [b] → [[a,b]]
     *
     * @see {@link https://ramdajs.com/docs#xprod}
    */
    xprod(ctx: RpsContext, opts: {}, ...params: any[]): Promise<any>;
    /**
     * @function zip
     * @memberof Basic
     * @example
     * log zip [1,2,3] ['a','b','c']
     *
     * @summary zip :: [a] → [b] → [[a,b]]
     *
     * @see {@link https://ramdajs.com/docs#zip}
    */
    zip(ctx: RpsContext, opts: {}, ...params: any[]): Promise<any>;
    /**
     * @function zip-obj
     * @memberof Basic
     * @example
     * log zip-obj ['a','b','c'] [1,2,3]
     *
     * @summary zip-obj :: [String] → [*] → {String:*}
     *
     * @see {@link https://ramdajs.com/docs#zipObj}
    */
    zipObj(ctx: RpsContext, opts: {}, ...params: any[]): Promise<any>;
    /**
     * @function zip-with
     * @memberof Basic
     *
     * @summary zip-with :: ((a,b) → c) [a] → [b] → [c]
     *
     * @see {@link https://ramdajs.com/docs#zipWith}
    */
    zipWith(ctx: RpsContext, opts: {}, ...params: any[]): Promise<any>;
    /**************************** Ramda (Object) ***********************************/
    /**
     * @function keys
     * @memberof Basic
     * @example
     * keys {a:1,b:2,c:3}
     *
     *
     * @see {@link https://ramdajs.com/docs#keys}
    */
    keys(ctx: RpsContext, opts: {}, ...params: any[]): Promise<any>;
    /**
     * @function assoc
     * @memberof Basic
     * @example
     * log assoc 'd' 4 {a:1,b:2,c:3}
     *
     * @summary assoc :: String → a → {k:v} → {k:v}
     *
     * @see {@link https://ramdajs.com/docs#assoc}
    */
    assoc(ctx: RpsContext, opts: {}, ...params: any[]): Promise<any>;
    /**
     * @function assoc-path
     * @memberof Basic
     * @example
     * log assoc-path ['c','d'] 4 {a:1,b:2,c:{d:0}}
     *
     * @summary assoc-path :: [Idx] → a → {a} → {a}
     *
     * @see {@link https://ramdajs.com/docs#assocPath}
    */
    assocPath(ctx: RpsContext, opts: {}, ...params: any[]): Promise<any>;
    /**
     * @function clone
     * @memberof Basic
     * @example
     * clone {a:1}
     *
     * @summary clone :: {*} → {*}
     *
     * @see {@link https://ramdajs.com/docs#clone}
    */
    clone(ctx: RpsContext, opts: {}, ...params: any[]): Promise<any>;
    /**
     * @function dissoc
     * @memberof Basic
     * @example
     * log dissoc 'b' {a:1,b:2,c:3}
     *
     * @summary dissoc :: String → {k:v} → {k:v}
     *
     * @see {@link https://ramdajs.com/docs#dissoc}
    */
    dissoc(ctx: RpsContext, opts: {}, ...params: any[]): Promise<any>;
    /**
     * @function dissoc-path
     * @memberof Basic
     * @example
     * log dissoc-path ['c','d'] 4 {a:1,b:2,c:{d:5}}
     *
     * @summary dissoc-path :: [Idx] → {k:v} → {k:v}
     *
     * @see {@link https://ramdajs.com/docs#dissocPath}
    */
    dissocPath(ctx: RpsContext, opts: {}, ...params: any[]): Promise<any>;
    /**
     * @function eq-props
     * @memberof Basic
     * @example
     * log eq-props 'a' {a:1} {a:1}
     *
     * @summary eq-props :: k → {k:v} → {k:v} →  Boolean
     *
     * @see {@link https://ramdajs.com/docs#eqProps}
    */
    eqProps(ctx: RpsContext, opts: {}, ...params: any[]): Promise<any>;
    /**
     * @function evolve
     * @memberof Basic
     * @example
     * log evolve {'firstName': trim } {'firstName': '  Tomato '}
     *
     * @summary evolve :: {k: (v → v)} → {k:v} → {k:v}
     *
     * @see {@link https://ramdajs.com/docs#evolve}
    */
    evolve(ctx: RpsContext, opts: {}, ...params: any[]): Promise<any>;
    /**
     * @function for-each-obj-indexed
     * @memberof Basic
     *
     * @summary for-each-obj-indexed :: ((a,String,StrMap a)  → Any) → StrMap a → StrMap a
     *
     * @see {@link https://ramdajs.com/docs#forEachObjIndexed}
    */
    forEachObjIndexed(ctx: RpsContext, opts: {}, ...params: any[]): Promise<any>;
    /**
     * @function has
     * @memberof Basic
     * @example
     * log has 'name' {name:'Alice'}
     *
     * @summary has :: s → {s:x} → Boolean
     *
     * @see {@link https://ramdajs.com/docs#has}
    */
    has(ctx: RpsContext, opts: {}, ...params: any[]): Promise<any>;
    /**
     * @function invert
     * @memberof Basic
     * @example
     * invert {first:'Alice',second:'allen',third:'Alice'}
     *
     * @summary invert :: {s:x} → {x:[s,...]}
     *
     * @see {@link https://ramdajs.com/docs#invert}
    */
    invert(ctx: RpsContext, opts: {}, ...params: any[]): Promise<any>;
    /**
     * @function invert-obj
     * @memberof Basic
     * @example
     * invert-obj {first:'Alice',second:'allen'}
     *
     * @summary invert-obj :: {s:x} → {x:s}
     *
     * @see {@link https://ramdajs.com/docs#invertObj}
    */
    invertObj(ctx: RpsContext, opts: {}, ...params: any[]): Promise<any>;
    /**
     * @function map-obj-indexed
     * @memberof Basic
     *
     * @summary map-obj-indexed :: ((*,String,Object)→ *) → Object → Object
     *
     * @see {@link https://ramdajs.com/docs#mapObjIndexed}
    */
    mapObjIndexed(ctx: RpsContext, opts: {}, ...params: any[]): Promise<any>;
    /**
     * @function merge
     * @memberof Basic
     * @example
     * merge {name:'fred',age:10} {age:40}
     *
     * @param {Object} object
     * @param {Object} object
     * @summary merge :: {k:v} → {k:v} → {k:v}
     *
     * @see {@link https://ramdajs.com/docs#merge}
    */
    merge(ctx: RpsContext, opts: {}, ...params: any[]): Promise<any>;
    /**
     * @function merge-deep-left
     * @memberof Basic
     *
     * @summary merge-deep-left :: {a} → {a} → {a}
     *
     * @see {@link https://ramdajs.com/docs#mergeDeepLeft}
    */
    mergeDeepLeft(ctx: RpsContext, opts: {}, ...params: any[]): Promise<any>;
    /**
     * @function merge-deep-right
     * @memberof Basic
     *
     * @summary merge :: {a} → {a} → {a}
     *
     * @see {@link https://ramdajs.com/docs#mergeDeepRight}
    */
    mergeDeepRight(ctx: RpsContext, opts: {}, ...params: any[]): Promise<any>;
    /**
     * @function merge-deep-with
     * @memberof Basic
     *
     * @summary merge-deep-with :: ((a,a) → a) → {a} → {a} → {a}
     *
     * @see {@link https://ramdajs.com/docs#mergeDeepWith}
    */
    mergeDeepWith(ctx: RpsContext, opts: {}, ...params: any[]): Promise<any>;
    /**
     * @function merge-with
     * @memberof Basic
     *
     * @summary merge-deep-with :: ((a,a) → a) → {a} → {a} → {a}
     *
     * @see {@link https://ramdajs.com/docs#mergeWith}
    */
    mergeWith(ctx: RpsContext, opts: {}, ...params: any[]): Promise<any>;
    /**
     * @function merge-with-key
     * @memberof Basic
     *
     * @summary merge-with-key :: ((String, a,a) → a) → {a} → {a} → {a}
     *
     * @see {@link https://ramdajs.com/docs#mergeWithKey}
    */
    mergeWithKey(ctx: RpsContext, opts: {}, ...params: any[]): Promise<any>;
    /**
     * @function obj-of
     * @memberof Basic
     * @example
     * obj-of 'must' ['a','b','c']
     *
     * @summary obj-of :: String → a → {String:a}
     *
     * @see {@link https://ramdajs.com/docs#objOf}
    */
    objOf(ctx: RpsContext, opts: {}, ...params: any[]): Promise<any>;
    /**
     * @function omit
     * @memberof Basic
     * @example
     * log omit ['a','b'] {a:1,b:2,c:3,d:4}
     *
     * @summary omit :: [String] → {String:*} → {String:*}
     *
     * @see {@link https://ramdajs.com/docs#omit}
    */
    omit(ctx: RpsContext, opts: {}, ...params: any[]): Promise<any>;
    /**
     * @function path
     * @memberof Basic
     * @example
     * log path ['a','b'] {a:1,{b:2}}
     *
     * @summary path :: [Idx] → {a} → a | undefined
     *
     * @see {@link https://ramdajs.com/docs#path}
    */
    path(ctx: RpsContext, opts: {}, ...params: any[]): Promise<any>;
    /**
     * @function path-or
     * @memberof Basic
     * @example
     * log path-or 'N/A' ['a','b'] {a:1,{b:2}}
     *
     * @summary path-or :: a → [Idx] → {a} → a
     *
     * @see {@link https://ramdajs.com/docs#pathOr}
    */
    pathOr(ctx: RpsContext, opts: {}, ...params: any[]): Promise<any>;
    /**
     * @function pick
     * @memberof Basic
     * @example
     * log pick ['a','b'] {a:1,b:2,c:3,d:4}
     *
     * @summary pick :: [k] → {k:v} → {k:v}
     *
     * @see {@link https://ramdajs.com/docs#pick}
    */
    pick(ctx: RpsContext, opts: {}, ...params: any[]): Promise<any>;
    /**
     * @function pick-all
     * @memberof Basic
     * @example
     * log pick-all ['a','b','g'] {a:1,b:2,c:3,d:4}
     *
     * @summary pick-all :: [k] → {k:v} → {k:v}
     *
     * @see {@link https://ramdajs.com/docs#pickAll}
    */
    pickAll(ctx: RpsContext, opts: {}, ...params: any[]): Promise<any>;
    /**
     * @function pick-by
     * @memberof Basic
     * @example
     * assign 'toUpperCase' ($val,$key) => equals $key to-upper $key
     * log pick-by $toUpperCase {a: 1, b: 2, A: 3, B: 4}
     *
     * @summary pick-by :: ((v,k) → Boolean) → {k:v} → {k:v}
     *
     * @see {@link https://ramdajs.com/docs#pickBy}
    */
    pickBy(ctx: RpsContext, opts: {}, ...params: any[]): Promise<any>;
    /**
     * @function project
     * @memberof Basic
     * @example
     * assign 'kids' [{name: 'Abby', age: 7, hair: 'blond', grade: 2},{name: 'Fred', age: 12, hair: 'brown', grade: 7}]
     * log project ['name', 'grade'] $kids
     *
     * @summary project :: [k] → [{k:v}] → [{k:v}]
     *
     * @see {@link https://ramdajs.com/docs#project}
    */
    project(ctx: RpsContext, opts: {}, ...params: any[]): Promise<any>;
    /**
     * @function prop
     * @memberof Basic
     * @example
     * prop 'x' {x:100}
     *
     * @summary prop :: s → {s:a} → a | undefined
     *
     * @see {@link https://ramdajs.com/docs#prop}
    */
    prop(ctx: RpsContext, opts: {}, ...params: any[]): Promise<any>;
    /**
     * @function prop-or
     * @memberof Basic
     * @example
     * log prop-or 'N/A' 'notfound' {x:100}
     *
     * @summary prop-or :: a → String → Object → a
     *
     * @see {@link https://ramdajs.com/docs#propOr}
    */
    propOr(ctx: RpsContext, opts: {}, ...params: any[]): Promise<any>;
    /**
     * @function props
     * @memberof Basic
     * @example
     * log props ['x','y'] {x:100,y:50}
     *
     * @summary props :: [k] → {k:v} → [v]
     *
     * @see {@link https://ramdajs.com/docs#props}
    */
    props(ctx: RpsContext, opts: {}, ...params: any[]): Promise<any>;
    /**
     * @function to-pairs
     * @memberof Basic
     * @example
     * to-pairs {a:1,b:2,c:3}
     *
     *
     * @see {@link https://ramdajs.com/docs#toPairs}
    */
    toPairs(ctx: RpsContext, opts: {}, ...params: any[]): Promise<any>;
    /**
     * @function values
     * @memberof Basic
     * @example
     * values {a:1,b:2,c:3}
     *
     * @summary values :: {k: v} → [v]
     *
     * @see {@link https://ramdajs.com/docs#values}
    */
    values(ctx: RpsContext, opts: {}, ...params: any[]): Promise<any>;
    /**
     * @function where
     * @memberof Basic
     *
     * @summary where :: {String: (* → Boolean} → {String:*} → Boolean
     *
     * @see {@link https://ramdajs.com/docs#where}
    */
    where(ctx: RpsContext, opts: {}, ...params: any[]): Promise<any>;
    /**
     * @function where-eq
     * @memberof Basic
     * @example
     * where-eq {a:1,b:2} {a:1,b:2}
     *
     * @summary where-eq :: {String:*} → {String:*} → Boolean
     *
     * @see {@link https://ramdajs.com/docs#whereEq}
    */
    whereEq(ctx: RpsContext, opts: {}, ...params: any[]): Promise<any>;
    /**************************** Ramda (Function) ***********************************/
    /**
     * @function __
     * @memberof Basic
     * @example
     * log call (replace "{name}" (__) "Hello, {name}!") 'Alice'
     *
     * @summary __
     *
     * @see {@link https://ramdajs.com/docs/#module.exports}
    */
    underscore(ctx: RpsContext, opts: {}): Promise<any>;
    /**
     * @function add-index
     * @memberof Basic
     * @example
     * add-index map | as 'idx'
     * log call $idx ($val,$idx)=> (identity `${$idx} - ${$val}`) ['f', 'o', 'o', 'b', 'a', 'r']
     *
     * @summary add-index :: ((a ... → b) ... → [a] → *) → (a ..., Int, [a] → b) ... → [a] → *)
     * @see {@link https://ramdajs.com/docs#addIndex}
    */
    addIndex(ctx: RpsContext, opts: {}, ...params: any[]): Promise<any>;
    /**
     * @function always
     * @memberof Basic
     * @example
     * log call (always 'Tee') 1
     *
     * @summary always :: a → (* → a)
     *
     * @see {@link https://ramdajs.com/docs#always}
    */
    always(ctx: RpsContext, opts: {}, ...params: any[]): Promise<any>;
    /**
     * @function ap
     * @memberof Basic
     * @example
     * log call (ap (concat) (to-upper)) 'Ramda'
     *
     * @summary ap :: [a → b] → [a] → [b]
     *
     * @see {@link https://ramdajs.com/docs#ap}
    */
    ap(ctx: RpsContext, opts: {}, ...params: any[]): Promise<any>;
    /**
     * @function apply
     * @memberof Basic
     * @example
     * log apply math-max [1,2,3,-99,42,6,7]
     *
     * @summary apply :: (*... → a) → [*] → a
     *
     * @see {@link https://ramdajs.com/docs#apply}
    */
    apply(ctx: RpsContext, opts: {}, ...params: any[]): Promise<any>;
    /**
     * @function apply-spec
     * @memberof Basic
     *
     * @see {@link https://ramdajs.com/docs#applySpec}
    */
    applySpec(ctx: RpsContext, opts: {}, ...params: any[]): Promise<any>;
    /**
     * @function apply-to
     * @memberof Basic
     * @example
     * log apply-to 42 (add 1)
     *
     * @summary apply-to :: a → (a → b) → b
     *
     * @see {@link https://ramdajs.com/docs#applyTo}
    */
    applyTo(ctx: RpsContext, opts: {}, ...params: any[]): Promise<any>;
    /**
     * @function ascend
     * @memberof Basic
     * @example
     * log sort (ascend prop "name") [{'name':'z'},{'name':'a'}]
     *
     * @summary ascend :: Ord b => (a → b) → a → a → Number
     *
     * @see {@link https://ramdajs.com/docs#ascend}
    */
    ascend(ctx: RpsContext, opts: {}, ...params: any[]): Promise<any>;
    /**
     * @function call
     * @memberof Basic
     * @example
     * call add 1 2
     *
     * @summary call :: (*... → a), *... → a
     *
     * @see {@link https://ramdajs.com/docs#call}
    */
    call(ctx: RpsContext, opts: {}, ...params: any[]): Promise<any>;
    /**
     * @function comparator
     * @memberof Basic
     * @example
     * comparator ($a,$b) => gt (prop 'age' $a) (prop 'age' $b) | as 'comp'
     * log sort $comp [{age:50},{age:10},{age:13},{age:25}]
     *
     * @summary comparator :: ((a,b) → Boolean) → ((a,b) → Number)
     *
     * @see {@link https://ramdajs.com/docs#comparator}
    */
    comparator(ctx: RpsContext, opts: {}, ...params: any[]): Promise<any>;
    /**
     * @function compose
     * @memberof Basic
     * @example
     * log call (compose (to-upper) (concat 'abc ')) 'introduce'
     *
     * @summary compose :: ((y → z), (x → y), ..., (o → p), ((a, b, ..., n) → o)) → ((a, b, ..., n) → z)
     *
     * @see {@link https://ramdajs.com/docs#compose}
    */
    compose(ctx: RpsContext, opts: {}, ...params: any[]): Promise<any>;
    /**
     * @function converge
     * @memberof Basic
     * @example
     * assign 'average' converge (divide) [sum, length]
     * log call $average [1, 2, 3, 4, 5, 6, 7]
     *
     * @summary converge :: ((x1, x2, ...) → z) → [((a, b, ...) → x1), ((a, b, ...) → x2), ...] → (a → b → ... → z)
     *
     * @see {@link https://ramdajs.com/docs#converge}
    */
    converge(ctx: RpsContext, opts: {}, ...params: any[]): Promise<any>;
    /**
     * @function descend
     * @memberof Basic
     * @example
     * log sort (descend prop "name") [{'name':'a'},{'name':'z'}]
     *
     * @summary descend :: Ord b => (a → b) → a → a → Number
     *
     * @see {@link https://ramdajs.com/docs#descend}
    */
    descend(ctx: RpsContext, opts: {}, ...params: any[]): Promise<any>;
    /**
     * @function empty
     * @memberof Basic
     * @example
     * log empty 'abc'
     *
     * @summary empty :: a → a
     *
     * @see {@link https://ramdajs.com/docs#empty}
    */
    empty(ctx: RpsContext, opts: {}, ...params: any[]): Promise<any>;
    /**
     * @function f
     * @memberof Basic
     * @example
     * log f
     *
     * @summary * → Boolean
     *
     * @see {@link https://ramdajs.com/docs#F}
    */
    F(ctx: RpsContext, opts: {}, ...params: any[]): Promise<any>;
    /**
     * @function identity
     * @memberof Basic
     * @example
     * log identity 1
     *
     * @summary identity :: a → a
     *
     * @see {@link https://ramdajs.com/docs#identity}
    */
    identity(ctx: RpsContext, opts: {}, ...params: any[]): Promise<any>;
    /**
     * @function juxt
     * @memberof Basic
     * @example
     * log call (juxt [math-min, math-max]) 3 4 9 -3
     *
     * @ juxt :: [(a, b, …, m) → n] → ((a, b, …, m) → [n])
     *
     * @see {@link https://ramdajs.com/docs#juxt}
    */
    juxt(ctx: RpsContext, opts: {}, ...params: any[]): Promise<any>;
    /**
     * @function nth-arg
     * @memberof Basic
     * @example
     * nth-arg 1 'a' 'b' 'c'
     *
     *
     * @summary nth-arg :: Number → *...  → *
     *
     * @see {@link https://ramdajs.com/docs#nthArg}
    */
    nthArg(ctx: RpsContext, opts: {}, ...params: any[]): Promise<any>;
    /**
     * @function of
     * @memberof Basic
     * @example
     * log of 42
     *
     *
     * @summary of :: a → [a]
     *
     * @see {@link https://ramdajs.com/docs#of}
    */
    of(ctx: RpsContext, opts: {}, ...params: any[]): Promise<any>;
    /**
     * @function pipe
     * @memberof Basic
     * @example
     * log call (pipe (negate) (inc)) 3
     *
     * @summary pipe :: (((a, b, …, n) → o), (o → p), …, (x → y), (y → z)) → ((a, b, …, n) → z)
     *
     * @see {@link https://ramdajs.com/docs#pipe}
    */
    pipe(ctx: RpsContext, opts: {}, ...params: any[]): Promise<any>;
    /**
     * @function t
     * @memberof Basic
     * @example
     * log t
     *
     * @summary * → Boolean
     *
     * @see {@link https://ramdajs.com/docs#T}
    */
    t(ctx: RpsContext, opts: {}, ...params: any[]): Promise<any>;
    /**
     * @function tap
     * @memberof Basic
     *
     * @summary tap :: (a → *) → a → a
     * @see {@link https://ramdajs.com/docs#tap}
    */
    tap(ctx: RpsContext, opts: {}, ...params: any[]): Promise<any>;
    /**
     * @function try-catch
     * @memberof Basic
     * @example
     * log call (try-catch (prop 'x') (f)) {x: true}
     *
     * @summary try-catch :: (…x → a) → ((e, …x) → a) → (…x → a)
     *
     * @see {@link https://ramdajs.com/docs#tryCatch}
    */
    tryCatch(ctx: RpsContext, opts: {}, ...params: any[]): Promise<any>;
    /**
     * @function use-with
     * @memberof Basic
     * @example
     * assign 'useWith' use-with (pow) [identity, identity]
     * log call $useWith 3 4
     *
     * @summary use-with :: ((x1, x2, …) → z) → [(a → x1), (b → x2), …] → (a → b → … → z)
     *
     * @see {@link https://ramdajs.com/docs#useWith}
    */
    useWith(ctx: RpsContext, opts: {}, ...params: any[]): Promise<any>;
    /**
     * @function unapply
     * @memberof Basic
     * @example
     * log unapply (stringify) 1 2 3
     *
     * @summary unapply :: ([*...] → a) → (*... → a)
     *
     * @see {@link https://ramdajs.com/docs#unapply}
    */
    unapply(ctx: RpsContext, opts: {}, ...params: any[]): Promise<any>;
    /**************************** Ramda (Relation) ***********************************/
    /**
    * @function clamp
    * @memberof Basic
    * @example
    * log clamp 1 10 15
    *
    *
    * @summary clamp :: Ord a => a → a → a → a
    *
    * @see {@link https://ramdajs.com/docs#clamp}
   */
    clamp(ctx: RpsContext, opts: {}, ...params: any[]): Promise<any>;
    /**
    * @function count-by
    * @memberof Basic
    * @example
    * log count-by (floor) [1.0,1.1,1.2,2.3,2.5]
    *
    * @summary count-by :: (a → String) → [a] → {*}
    *
    * @see {@link https://ramdajs.com/docs#countBy}
   */
    countBy(ctx: RpsContext, opts: {}, ...params: any[]): Promise<any>;
    /**
    * @function difference
    * @memberof Basic
    * @example
    * log difference [1,2,3,4] [3,4,5,6]
    *
    * @summary difference :: [*] → [*] → [*]
    *
    * @see {@link https://ramdajs.com/docs#difference}
   */
    difference(ctx: RpsContext, opts: {}, ...params: any[]): Promise<any>;
    /**
    * @function difference-with
    * @memberof Basic
    * @example
    * assign 'cmp' eq-props 'a'
    * log difference-with $cmp [{a: 1}, {a: 2}, {a: 3}] [{a: 3}, {a: 4}]
    *
    * @summary difference-with ((a,a) → Boolean) → [a] → [a] → [a]
    *
    * @see {@link https://ramdajs.com/docs#differenceWith}
   */
    differenceWith(ctx: RpsContext, opts: {}, ...params: any[]): Promise<any>;
    /**
    * @function eq-by
    * @memberof Basic
    * @example
    * log eq-by (abs) 5 -5
    *
    * @summary (a → b) → a → a → Boolean
    *
    * @see {@link https://ramdajs.com/docs#eqBy}
   */
    eqBy(ctx: RpsContext, opts: {}, ...params: any[]): Promise<any>;
    /**
    * @function equals
    * @memberof Basic
    * @example
    * equals 5 5
    *
    * @summary equals :: a → b → Boolean
    *
    * @see {@link https://ramdajs.com/docs#equals}
   */
    equals(ctx: RpsContext, opts: {}, ...params: any[]): Promise<any>;
    /**
    * @function gt
    * @memberof Basic
    * @example
    * log gt 2 1
    *
    *
    * @summary Ord a => a → a → Boolean
    *
    * @see {@link https://ramdajs.com/docs#gt}
   */
    gt(ctx: RpsContext, opts: {}, ...params: any[]): Promise<any>;
    /**
    * @function gte
    * @memberof Basic
    * @example
    * log gte 2 2
    *
    * @summary Ord a => a → a → Boolean
    *
    * @see {@link https://ramdajs.com/docs#gte}
   */
    gte(ctx: RpsContext, opts: {}, ...params: any[]): Promise<any>;
    /**
    * @function identical
    * @memberof Basic
    * @example
    * log identical 1 1
    *
    *
    * @summary identical :: a → a → Boolean
    *
    * @see {@link https://ramdajs.com/docs#identical}
   */
    identical(ctx: RpsContext, opts: {}, ...params: any[]): Promise<any>;
    /**
    * @function inner-join
    * @memberof Basic
    * @example
    * assign 'idMatch' ($record, $id) => equals $id prop 'id' $record
    * log inner-join $idMatch [
    *    {id: 824, name: 'Richie Furay'},
    *     {id: 956, name: 'Dewey Martin'},
    *     {id: 313, name: 'Bruce Palmer'},
    *     {id: 456, name: 'Stephen Stills'},
    *     {id: 177, name: 'Neil Young'}] [177, 456, 999]
    *
    * @summary inner-join :: ((a,b) → Boolean) → [a] → [b] → [a]
    *
    * @see {@link https://ramdajs.com/docs#innerJoin}
   */
    innerJoin(ctx: RpsContext, opts: {}, ...params: any[]): Promise<any>;
    /**
    * @function intersection
    * @memberof Basic
    * @example
    * log intersection [1,2,3,4] [3,4,5,6]
    *
    * @summary intersection :: [ * ] → [ * ] → [ * ]
    *
    * @see {@link https://ramdajs.com/docs#intersection}
   */
    intersection(ctx: RpsContext, opts: {}, ...params: any[]): Promise<any>;
    /**
    * @function lt
    * @memberof Basic
    * @example
    * log lt 2 1
    *
    *
    * @summary lt :: Ord a => a → a → Boolean
    *
    * @see {@link https://ramdajs.com/docs#lt}
   */
    lt(ctx: RpsContext, opts: {}, ...params: any[]): Promise<any>;
    /**
    * @function lte
    * @memberof Basic
    * @example
    * lte 2 2
    *
    *
    * @summary lte :: Ord a => a → a → Boolean
    *
    * @see {@link https://ramdajs.com/docs#lte}
   */
    lte(ctx: RpsContext, opts: {}, ...params: any[]): Promise<any>;
    /**
    * @function max
    * @memberof Basic
    * @example
    * max 2 4
    *
    * @param {*} any
    * @param {*} any
    *
    * @summary Ord a => a → a → a
    *
    * @see {@link https://ramdajs.com/docs#max}
   */
    max(ctx: RpsContext, opts: {}, ...params: any[]): Promise<any>;
    /**
    * @function max-by
    * @memberof Basic
    * @example
    * log max-by (abs) -3 2 -1
    *
    * @summary max-by :: Ord a => (a → b) → a → a → a
    *
    * @see {@link https://ramdajs.com/docs#maxBy}
   */
    maxBy(ctx: RpsContext, opts: {}, ...params: any[]): Promise<any>;
    /**
    * @function min
    * @memberof Basic
    * @example
    * log min 10 5
    *
    * @summary min :: Ord a => a → a → a
    *
    * @see {@link https://ramdajs.com/docs#min}
   */
    min(ctx: RpsContext, opts: {}, ...params: any[]): Promise<any>;
    /**
    * @function min-by
    * @memberof Basic
    * @example
    * log min-by (abs) -3 2 -1
    *
    * @summary min-by :: Ord a => (a → b) → a → a → a
    *
    * @see {@link https://ramdajs.com/docs#minBy}
   */
    minBy(ctx: RpsContext, opts: {}, ...params: any[]): Promise<any>;
    /**
    * @function path-eq
    * @memberof Basic
    *
    * @summary path-eq :: [Idx] → a → {a} → Boolean
    *
    * @see {@link https://ramdajs.com/docs#pathEq}
   */
    pathEq(ctx: RpsContext, opts: {}, ...params: any[]): Promise<any>;
    /**
    * @function prop-eq
    * @memberof Basic
    * @example
    * log prop-eq 'a' 2 {'a': 2}
    *
    * @summary prop-eq :: String → a → Object → Boolean
    *
    * @see {@link https://ramdajs.com/docs#propEq}
   */
    propEq(ctx: RpsContext, opts: {}, ...params: any[]): Promise<any>;
    /**
    * @function sort-by
    * @memberof Basic
    * @example
    * log sort-by (prop 0) [[-1,1] , [-2,2] , [-3,3]]
    *
    * @summary sort-by :: Ord b => (a → b) → [a] → [a]
    *
    * @see {@link https://ramdajs.com/docs#sortBy}
   */
    sortBy(ctx: RpsContext, opts: {}, ...params: any[]): Promise<any>;
    /**
    * @function sort-with
    * @memberof Basic
    *
    * @summary sort-with :: [(a,a) → Number] → [a] → [a]
    *
    * @see {@link https://ramdajs.com/docs#sortWith}
   */
    sortWith(ctx: RpsContext, opts: {}, ...params: any[]): Promise<any>;
    /**
    * @function symmetric-difference
    * @memberof Basic
    * @example
    * symmetric-difference [1,2,3,4] [3,4,5,6,7]
    *
    * @summary symmetric-difference :: [*] → [*] → [*]
    *
    * @see {@link https://ramdajs.com/docs#symmetricDifference}
   */
    symmetricDifference(ctx: RpsContext, opts: {}, ...params: any[]): Promise<any>;
    /**
    * @function symmetric-difference-with
    * @memberof Basic
    *
    * @summary symmetric-difference-with :: ((a,a) → Boolean) → [a] → [a] → [a]
    *
    * @see {@link https://ramdajs.com/docs#symmetricDifferenceWith}
   */
    symmetricDifferenceWith(ctx: RpsContext, opts: {}, ...params: any[]): Promise<any>;
    /**
    * @function union
    * @memberof Basic
    * @example
    * log union [1,2,3] [2,3,4]
    *
    * @summary union :: [*] → [*] → [*]
    *
    * @see {@link https://ramdajs.com/docs#union}
   */
    union(ctx: RpsContext, opts: {}, ...params: any[]): Promise<any>;
    /**
    * @function union-with
    * @memberof Basic
    * @example
    * union-with (eq-by prop 'a') [{a:1}] [{a:1}]
    *
    *
    * @summary union-with :: ((a,a) → Boolean) → [*] → [*] → [*]
    *
    * @see {@link https://ramdajs.com/docs#unionWith}
   */
    unionWith(ctx: RpsContext, opts: {}, ...params: any[]): Promise<any>;
    /**************************** Ramda (Logic) ***********************************/
    /**
     * @function all-pass
     * @memberof Basic
     * @example
     * log call ( all-pass [ (prop-eq 'name' 'ABC'),(prop-eq 'type' 'org') ] ) {name:'ABC','type':'org'}
     *
     * @summary all-pass :: [(*...) → Boolean] → (*... → Boolean)
     *
     * @see {@link https://ramdajs.com/docs#allPass}
    */
    allPass(ctx: RpsContext, opts: {}, ...params: any[]): Promise<any>;
    /**
     * @function and
     * @memberof Basic
     * @example
     * log and true false
     *
     *
     * @summary and :: a → b → a | b
     *
     * @see {@link https://ramdajs.com/docs#and}
    */
    and(ctx: RpsContext, opts: {}, ...params: any[]): Promise<any>;
    /**
     * @function any-pass
     * @memberof Basic
     * @example
     * log call ( any-pass [ (prop-eq 'name' 'ABC'),(prop-eq 'type' 'org') ] ) {name:'K','type':'org'}
     *
     * @summary any-pass :: [(*... → Boolean)] → (*... → Boolean)
     *
     * @see {@link https://ramdajs.com/docs#anyPass}
    */
    anyPass(ctx: RpsContext, opts: {}, ...params: any[]): Promise<any>;
    /**
     * @function both
     * @memberof Basic
     * @example
     * log call ( both (gt (__) 10) (lt (__) 20) ) 15
     *
     * @summary both :: (*... → Boolean) → (*... → Boolean) → (*... → Boolean)
     *
     * @see {@link https://ramdajs.com/docs#both}
    */
    both(ctx: RpsContext, opts: {}, ...params: any[]): Promise<any>;
    /**
     * @function complement
     * @memberof Basic
     * @example
     * log call (complement (is-nil)) 1
     *
     * @summary complement :: (*... → *) → (*... → Boolean)
     *
     * @see {@link https://ramdajs.com/docs#complement}
    */
    complement(ctx: RpsContext, opts: {}, ...params: any[]): Promise<any>;
    /**
     * @function cond
     * @memberof Basic
     *
     * @see {@link https://ramdajs.com/docs#cond}
    */
    cond(ctx: RpsContext, opts: {}, ...params: any[]): Promise<any>;
    /**
     * @function default-to
     * @memberof Basic
     * @example
     * log default-to 10 null
     *
     * @summary a → b → a | b
     *
     * @see {@link https://ramdajs.com/docs#defaultTo}
    */
    defaultTo(ctx: RpsContext, opts: {}, ...params: any[]): Promise<any>;
    /**
     * @function either
     * @memberof Basic
     *
     * @summary (*... → Boolean) → (*... → Boolean) → (*... → Boolean)
     *
     * @see {@link https://ramdajs.com/docs#either}
    */
    either(ctx: RpsContext, opts: {}, ...params: any[]): Promise<any>;
    /**
     * @function if-else
     * @memberof Basic
     * @example
     * assign 'ifElse' if-else (has 'count') (over (lens-prop 'count') inc) (assoc 'count' 1)
     * log call $ifElse {}
     * log call $ifElse {'count':4}
     *
     * @summary if-else :: (*… → Boolean) → (*… → *) → (*… → *) → (*… → *)
     *
     * @see {@link https://ramdajs.com/docs#ifElse}
    */
    ifElse(ctx: RpsContext, opts: {}, ...params: any[]): Promise<any>;
    /**
     * @function is-empty
     * @memberof Basic
     * @example
     * log is-empty []
     *
     * @summary is-empty :: a → Boolean
     *
     * @see {@link https://ramdajs.com/docs#isEmpty}
    */
    isEmpty(ctx: RpsContext, opts: {}, ...params: any[]): Promise<any>;
    /**
     * @function not
     * @memberof Basic
     * @example
     * log not 1
     *
     * @summary not :: * → Boolean
     *
     * @see {@link https://ramdajs.com/docs#not}
    */
    not(ctx: RpsContext, opts: {}, ...params: any[]): Promise<any>;
    /**
     * @function or
     * @memberof Basic
     * @example
     * log or true false
     *
     * @summary or :: a → b → a | b
     *
     * @see {@link https://ramdajs.com/docs#or}
    */
    or(ctx: RpsContext, opts: {}, ...params: any[]): Promise<any>;
    /**
     * @function path-satisfies
     * @memberof Basic
     *
     * @summary path-satisfies ::
     * (a → Boolean) → [Idx] → {a} → Boolean
     * Idx = String | Int
     *
     * @see {@link https://ramdajs.com/docs#pathSatisfies}
    */
    pathSatisfies(ctx: RpsContext, opts: {}, ...params: any[]): Promise<any>;
    /**
     * @function prop-satisfies
     * @memberof Basic
     *
     * @summary prop-satisfies :: (a → Boolean) → String → {String: a} → Boolean
     *
     * @see {@link https://ramdajs.com/docs#propSatisfies}
    */
    propSatisfies(ctx: RpsContext, opts: {}, ...params: any[]): Promise<any>;
    /**
     * @function unless
     * @memberof Basic
     * @example
     * log call (unless (is-nil) (inc)) 1
     *
     * @summary unless :: (a → Boolean) → (a → a) → a → a
     *
     * @see {@link https://ramdajs.com/docs#unless}
    */
    unless(ctx: RpsContext, opts: {}, ...params: any[]): Promise<any>;
    /**
     * @function until
     * @memberof Basic
     * @example
     * log call (until (gt (__) 100) (multiply 2) ) 1
     *
     * @summary until :: (a → Boolean) → (a → a) → a → a
     *
     * @see {@link https://ramdajs.com/docs#until}
    */
    until(ctx: RpsContext, opts: {}, ...params: any[]): Promise<any>;
    /**
     * @function when
     * @memberof Basic
     * @example
     * assign 'truncate' when (prop-satisfies (gt __ 10) 'length') (pipe (take 10) (append '...') (join '') )
     * log call $truncate '0123456789ABC'
     *
     * @summary when :: (a → Boolean) → (a → a) → a → a
     *
     * @see {@link https://ramdajs.com/docs#when}
    */
    when(ctx: RpsContext, opts: {}, ...params: any[]): Promise<any>;
    /**************************** Ramda (String) ***********************************/
    /**
    * @function trim
    * @memberof Basic
    * @example
    * log trim " hello   "
    *
    * @summary String → String
    *
    * @see {@link https://ramdajs.com/docs#trim}
   */
    trim(ctx: RpsContext, opts: {}, ...params: any[]): Promise<any>;
    /**
    * @function match
    * @memberof Basic
    * @example
    * log match (regexp "a") "banana"
    *
    * @summary match :: RegExp → String → [String | Undefined]
    *
    * @see {@link https://ramdajs.com/docs#match}
   */
    match(ctx: RpsContext, opts: {}, ...params: any[]): Promise<any>;
    /**
    * @function replace
    * @memberof Basic
    * @example
    * log replace "a" "b" "banana"
    *
    * @summary :: replace :: RegExp|String → String → String → String
    *
    * @see {@link https://ramdajs.com/docs#replace}
   */
    replace(ctx: RpsContext, opts: {}, ...params: any[]): Promise<any>;
    /**
    * @function split
    * @memberof Basic
    * @example
    * log split ',' "hello, world, hi"
    *
    * @summary split :: (String | RegExp) → String → [String]
    *
    * @see {@link https://ramdajs.com/docs#split}
   */
    split(ctx: RpsContext, opts: {}, ...params: any[]): Promise<any>;
    /**
    * @function test
    * @memberof Basic
    * @example
    * log test (regexp "p") "banana"
    *
    * @summary test :: RegExp → String → Boolean
    *
    * @see {@link https://ramdajs.com/docs#test}
   */
    test(ctx: RpsContext, opts: {}, ...params: any[]): Promise<any>;
    /**
    * @function to-lower
    * @memberof Basic
    * @example
    * log to-lower "ABZ"
    *
    * @summary to-lower :: String → String
    *
    * @see {@link https://ramdajs.com/docs#toLower}
   */
    toLower(ctx: RpsContext, opts: {}, ...params: any[]): Promise<any>;
    /**
    * @function to-string
    * @memberof Basic
    * @example
    * log to-string 100
    *
    * @summary to-string :: * → String
    *
    * @see {@link https://ramdajs.com/docs#toString}
   */
    toString(ctx: RpsContext, opts: {}, ...params: any[]): Promise<any>;
    /**
     *
    * @function to-upper
    * @memberof Basic
    * @example
    * log to-upper "abc"
    *
    * @summary to-upper :: * → String
    *
    * @see {@link https://ramdajs.com/docs#toUpper}
   */
    toUpper(ctx: RpsContext, opts: {}, ...params: any[]): Promise<any>;
    /**************************** Ramda (Math) ***********************************/
    /**
    * @function add
    * @memberof Basic
    * @example
    * log add 3 4
    *
    * @summary add :: Number → Number → Number
    * @see {@link https://ramdajs.com/docs#add}
   */
    add(ctx: RpsContext, opts: {}, ...params: any[]): Promise<any>;
    /**
    * @function dec
    * @memberof Basic
    * @example
    * log dec 42
    *
    * @summary dec :: Number → Number
    *
    * @see {@link https://ramdajs.com/docs#dec}
   */
    dec(ctx: RpsContext, opts: {}, ...params: any[]): Promise<any>;
    /**
    * @function divide
    * @memberof Basic
    * @example
    * log divide 6 2
    *
    * @summary divide :: Number → Number → Number
    *
    * @see {@link https://ramdajs.com/docs#divide}
   */
    divide(ctx: RpsContext, opts: {}, ...params: any[]): Promise<any>;
    /**
    * @function inc
    * @memberof Basic
    * @example
    * log inc 41
    *
    * @summary inc :: Number → Number
    *
    * @see {@link https://ramdajs.com/docs#inc}
   */
    inc(ctx: RpsContext, opts: {}, ...params: any[]): Promise<any>;
    /**
    * @function math-mod
    * @memberof Basic
    * @example
    * log math-mod 17 5
    *
    * @summary math-mod :: Number → Number → Number
    *
    * @see {@link https://ramdajs.com/docs#mathMod}
   */
    mathMod(ctx: RpsContext, opts: {}, ...params: any[]): Promise<any>;
    /**
    * @function mean
    * @memberof Basic
    *
    * @see {@link https://ramdajs.com/docs#mean}
   */
    mean(ctx: RpsContext, opts: {}, ...params: any[]): Promise<any>;
    /**
    * @function median
    * @memberof Basic
    *
    * @see {@link https://ramdajs.com/docs#median}
   */
    median(ctx: RpsContext, opts: {}, ...params: any[]): Promise<any>;
    /**
    * @function modulo
    * @memberof Basic
    * @example
    * log modulo 17 5
    *
    * @summary modulo :: Number → Number → Number
    *
    * @see {@link https://ramdajs.com/docs#modulo}
   */
    modulo(ctx: RpsContext, opts: {}, ...params: any[]): Promise<any>;
    /**
    * @function multiply
    * @memberof Basic
    * @example
    * log multiply 4 3
    *
    * @summary multiply :: Number → Number → Number
    *
    * @see {@link https://ramdajs.com/docs#multiply}
   */
    multiply(ctx: RpsContext, opts: {}, ...params: any[]): Promise<any>;
    /**
    * @function negate
    * @memberof Basic
    * @example
    * log negate -42
    *
    * @summary negate :: Number → Number
    *
    * @see {@link https://ramdajs.com/docs#negate}
   */
    negate(ctx: RpsContext, opts: {}, ...params: any[]): Promise<any>;
    /**
    * @function product
    * @memberof Basic
    * @example
    * log product [2,3]
    *
    * @summary product :: [Number] → Number
    *
    * @see {@link https://ramdajs.com/docs#product}
   */
    product(ctx: RpsContext, opts: {}, ...params: any[]): Promise<any>;
    /**
    * @function subtract
    * @memberof Basic
    * @example
    * log subtract 10 4
    *
    * @summary subtract :: Number → Number → Number
    *
    * @see {@link https://ramdajs.com/docs#subtract}
   */
    subtract(ctx: RpsContext, opts: {}, ...params: any[]): Promise<any>;
    /**
    * @function sum
    * @memberof Basic
    * @example
    * log sum [3 5]
    *
    * @summary sum :: [Number] → Number
    *
    * @see {@link https://ramdajs.com/docs#sum}
   */
    sum(ctx: RpsContext, opts: {}, ...params: any[]): Promise<any>;
    /**************************** Ramda (Type) ***********************************/
    /**
    * @function is
    * @memberof Basic
    * @example
    * log to-string is Array [1,2,3]
    *
    * @summary is :: (* → {*}) → a → Boolean
    *
    * @see {@link https://ramdajs.com/docs#is}
   */
    is(ctx: RpsContext, opts: {}, ...params: any[]): Promise<any>;
    /**
    * @function is-nil
    * @memberof Basic
    * @example
    * log is-nil []
    *
    * @summary is-nil :: * → Boolean
    *
    * @see {@link https://ramdajs.com/docs#isNil}
   */
    isNil(ctx: RpsContext, opts: {}, ...params: any[]): Promise<any>;
    /**
    * @function prop-is
    * @memberof Basic
    * @example
    * log to-string prop-is Number 'x' {x:1}
    *
    * @summary prop-is :: Type → String → Object → Boolean
    *
    * @see {@link https://ramdajs.com/docs#propIs}
   */
    propIs(ctx: RpsContext, opts: {}, ...params: any[]): Promise<any>;
    /**
    * @function type
    * @memberof Basic
    * @example
    * log type 1
    *
    * @summary type :: (* → {*}) → String
    *
    * @see {@link https://ramdajs.com/docs#type}
   */
    type(ctx: RpsContext, opts: {}, ...params: any[]): Promise<any>;
    /**************************** Ramda (Customize) ***********************************/
    /**
    * @function r
    * @memberof Basic
    *
   */
    r(ctx: RpsContext, opts: {}, functionName: string): Promise<any>;
    /**
    * @function regexp
    * @memberof Basic
    *
    * @param {RegExp} regexp regular expression
    *
    * @summary regexp
    *
   */
    regexp(ctx: RpsContext, opts: {}, regexp: string): Promise<RegExp>;
}
